/*

	Converted to PIC code on 12/17/09 by Jess Lewis	
	Project for SPI messaging to a MAX7456 OSC chip.
 * Used PIC16F688 device. */

//Try changing the delay in the SPI routines from 1us to no delay.  Code works fine with the delay but may not work without it.

#include <stdio.h>
#include <stdlib.h>
#include <htc.h>
#include "shiftdata.h"
#include "DS1302.h"
#include "max7456.h"


__CONFIG(FOSC_INTOSCIO & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_OFF & IESO_OFF & FCMEN_OFF);	

#define _XTAL_FREQ		4000000 //used for internal delay functions

#define TIME_SET_DDR 		TRISC3	
#define TIME_CHANGE_DDR 	TRISC4	
#define TIME_SET 			RC3
#define TIME_CHANGE	 		RC4

unsigned char x;
unsigned char state = 0;  //state 0 = display time; state 1 = set time
unsigned char blink = 0;
unsigned char currentchar = 0; //current character to update


unsigned char checkTimeSetPB(void);
unsigned char checkTimeChangePB(void);
void config_io(void);
void initOSD(void);
void outtoOSD(void);


unsigned char checkTimeSetPB(void)
{
	if(TIME_SET == 1)
	{
		__delay_ms(10);

		if (TIME_SET == 1)
		{
			return 1;
		}
	}

	//button must not be pressed
	return 0;
}


unsigned char checkTimeChangePB(void)
{
	if(TIME_CHANGE == 1)
	{	
		__delay_ms(10);

		if (TIME_CHANGE == 1)
		{
			return 1;
		}
	}

	//button must not be pressed
	return 0;
}


void config_io(void)
{
	shiftdataconfig_io(); //Setup IO pins and defaults
	DS1302Config_IO();

	//all analogs set to digital and all comparators turned off in shiftdata code
	//otherwise it would need to be done for the pushbuttons to work
	TIME_SET_DDR = 1;  	//pushbutton to set the time to the current selection
	TIME_CHANGE_DDR = 1;  //pushbutton to change a time value
}


void initOSD(void)
{
	spiWriteReg(VM0_WRITE,VM0_01_EnVideo);  //reset display of OSD image
	__delay_ms(1);

	spiWriteReg(VM0_WRITE,VM0_08_EnOSD);  //enable display of OSD image
	__delay_ms(1);

	spiWriteReg(DMM_WRITE, 0x04); //clear display
	__delay_ms(20); //must delay at least 20us

	//automatic black level control, have to read, augment and rewrite
	//The data sheet is rather specific about this
	x = spiReadReg(OSDBL_READ);
	__delay_ms(1);	
	x &= 0xEF;
	__delay_ms(1);
	spiWriteReg(OSDBL_WRITE,x);
	__delay_ms(1);	
	
	spiWriteReg(DMAH_WRITE,0x01);//DMAH
}

void outtoOSD(void)
{	
	read_DS1302TimeDate();  //set up date and time string for display output
	
	if ((state == 1) & (blink == 1))
	{
		switch(currentchar)
		{
			case 0: 	//month 
				datetimestring[0] = 0;
				datetimestring[1] = 0;			
				break;
			case 1:		//date  
				datetimestring[2] = 0;
				datetimestring[3] = 0;
				break;	
			case 2: 	//year
				datetimestring[6] = 0;
				datetimestring[7] = 0;	
				break; 
			case 3:		//hours
				datetimestring[8] = 0;
				datetimestring[9] = 0;		
				break;
			case 4:		//mins
				datetimestring[10] = 0;
				datetimestring[11] = 0;	
				break;
			case 5:		//secs
				datetimestring[12] = 0;
				datetimestring[13] = 0;	
				break;
		}
		blink = 0;
	}
	else if (state == 1)
	{
		blink = 1;
	}

	x	= 32;  //position of first character displayed - top left of screen

	/* By setting the DMAH bit to 1, this means that the minimum screen address that any character will be written to will be 256.
	   Whatever x is assigned to will be added to 256 to obtain the screen address if x is used for the DMAL.
	   In summary, to write to any of the first 256 addresses on the screen, just assign the number to DMAL and assign a 0 to DMAH.
	   For the lower half of the screen, set DMAH to 1 which happens to be in row 9 of 13 available rows.	
	*/

	spiWriteReg(DMAL_WRITE,x);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[0]);  //month tens

	spiWriteReg(DMAL_WRITE,x+1);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[1]);  //month ones

	spiWriteReg(DMAL_WRITE,x+2);//DMAL
	spiWriteReg(DMDI_WRITE,0x47); // slash separator

	spiWriteReg(DMAL_WRITE,x+3);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[2]);  //day tens

	spiWriteReg(DMAL_WRITE,x+4);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[3]);  //day ones

	spiWriteReg(DMAL_WRITE,x+5);//DMAL
	spiWriteReg(DMDI_WRITE,0x47);  //slash separator

	spiWriteReg(DMAL_WRITE,x+6);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[4]); //year thousands

	spiWriteReg(DMAL_WRITE,x+7);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[5]);  //year hundreds

	spiWriteReg(DMAL_WRITE,x+8);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[6]); //year tens

	spiWriteReg(DMAL_WRITE,x+9);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[7]); //year ones

	spiWriteReg(DMAL_WRITE,x+10);//DMAL
	spiWriteReg(DMDI_WRITE,0x00); //space separator

	spiWriteReg(DMAL_WRITE,x+11);//DMAL
	spiWriteReg(DMDI_WRITE,0x00); //space separator

	spiWriteReg(DMAL_WRITE,x+12);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[8]); //hours tens

	spiWriteReg(DMAL_WRITE,x+13);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[9]); //hours ones
	
	spiWriteReg(DMAL_WRITE,x+14);//DMAL
	spiWriteReg(DMDI_WRITE,0x44); //colon separator

	spiWriteReg(DMAL_WRITE,x+15);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[10]); //minutes tens
	
	spiWriteReg(DMAL_WRITE,x+16);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[11]); //minutes ones
	
	spiWriteReg(DMAL_WRITE,x+17);//DMAL
	spiWriteReg(DMDI_WRITE,0x44); //colon seperator
	
	spiWriteReg(DMAL_WRITE,x+18);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[12]);  //seconds tens
	
	spiWriteReg(DMAL_WRITE,x+19);//DMAL
	spiWriteReg(DMDI_WRITE,datetimestring[13]); //seconds ones
	
	spiWriteReg(VM0_WRITE,OSD_EN);	//enable display of OSD image

	__delay_ms(40); //must delay at least 20us
}


int main (void)
{
	unsigned char temp = 0;

    config_io(); //Setup IO pins and defaults

	__delay_ms(10); //pause a moment
	
	initOSD();  //initialize display
	initialize_DS1302();

	//this delay is necessary to allow the oscillator to settle down 
	for(unsigned char i = 0; i < 5; ++i)
	{
		__delay_ms(200); //pause a moment
	}

	while(1)
	{
		if(state == 1)
		{
			if (checkTimeChangePB() == 1)  
			{			
				DS1302ChangeDT(currentchar); //change month only for testing. 
			}
			
			if (checkTimeSetPB() == 1)
			{
				currentchar ++;
				if(currentchar > 5)
				{
					//change back to display mode
					state = 0;
					blink = 0;
					currentchar = 0;
				}
			}
		}
		else
		{
			//enter time change mode
			if (checkTimeSetPB() == 1)
			{
				state = 1;	
				currentchar = 0;	
				blink = 1;
			}
		}

		//display time
		outtoOSD();
		
		__delay_ms(100);
	} //end while	
}
